<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

	<!-- Sämtliche Leerzeilen und Zeilenumbrüche löschen -->	
	<xsl:strip-space elements="*"/>
	<!-- XML Output neu Formatieren -->
	<xsl:output method="xml" encoding="UTF-8" indent="yes"/>

	
	<xsl:variable name="true" select="'true'"/>
	<xsl:variable name="false" select="'false'"/>



	<!-- forms ist das Rootelement, es kann folgende Subelemtente haben: meta, quittung, formset, notes -->
 	<xsl:template name="root" match="forms">
 		<xsl:element name="forms">
	 		<xsl:apply-templates select="child::node()"/>
 		</xsl:element>
 	</xsl:template>
	
	<!-- Metadaten nicht importieren -->
	<xsl:template name="meta" match="meta"/>
	<!-- Quitungsdaten nicht importieren -->
	<xsl:template name="quittung" match="quittung"/>
	<!-- Notizen nicht importieren -->
	<xsl:template name="notes" match="notes"/>

	<!-- formset hat nur form als Subelement -->
	<xsl:template name="formset" match="formset">
		<xsl:variable name="formsetId">
			<xsl:call-template name="checkId">
				<xsl:with-param name="id" select="@id"/>
				<xsl:with-param name="isInInstancePart" select="$false"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:element name="{$formsetId}">
			<xsl:apply-templates select="child::node()"/>
		</xsl:element>		
	</xsl:template>
	
	
	<!-- 
	form (eine Formularseite) kann folgende Subelemente haben: 
	dictionary, table, view, merge, voucher-table, list, instance-list, instance-reference-list, 
	integer, float, date, string, text, boolean, barcode-group, version, current-time, instance-part
	-->
 	<xsl:template name="form" match="form">
		<xsl:variable name="formId">
			<xsl:call-template name="checkId">
				<xsl:with-param name="id" select="@id"/>
				<xsl:with-param name="isInInstancePart" select="$false"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:element name="{$formId}">
			<!-- Als erstes prüfen, ob es sich um ein instanzierbares Formular handel, falls ja muss das Flag 'isInInstancePart' auf 'true' gesetz werden!  -->
 			<xsl:for-each select="instance-part">
				<xsl:choose>
					<!-- Das erste ist das Template, und wird nicht importiert  -->
					<xsl:when test="position() = '1'"/>
					<xsl:otherwise>
						<xsl:apply-templates select=".">
							<xsl:with-param name="isInInstancePart" select="$true"/>							
						</xsl:apply-templates>	
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>
 			
 			<!-- Da hier auch die instance-part's nochmals kommen, muss das Flag 'isInInstancePart' auf 'false' gesetz werden! -->
 			<xsl:apply-templates select="child::node()" >
				<xsl:with-param name="isInInstancePart" select="$false"/>
			</xsl:apply-templates>
				
 			</xsl:element>		
	</xsl:template>
	
	
	<!-- Elemente welche nicht importiet werden können, da sie erst zur Laufzeit gefüllt werden -->
	<xsl:template match="instance-list | view | merge | version | current-time | voucher-table" />
	<!-- Elemnte für welche ein Import keinen Sinn macht... -->
	<xsl:template match="barcode-group"/>


	<!--
	Der instance-part soll nur dann übernommen werden, wenn das Flag 'isInInstancePart' auf 'true' gesetzt ist.
	Andernfalls wird er nicht mehr übernommen, weil er bereist in einem vorherigehenden Loop übernommen wurde.
	 -->
	<xsl:template name="instance-part" match="instance-part">
		<xsl:param name="isInInstancePart" />
		<xsl:choose>
			<xsl:when test="$isInInstancePart = $true">
		 		<xsl:variable name="parentId" select="../@id" />
				<xsl:element name="{concat($parentId, '_InstanceTable')}" >
			 		<xsl:apply-templates select="child::node()">
			 			<xsl:with-param name="isInInstancePart" select="$true"/>
			 		</xsl:apply-templates>
		 		</xsl:element>
			</xsl:when>
			<xsl:otherwise/>
		</xsl:choose>
	</xsl:template>
	
	
	<!--
	Tabellen müssen komplett umsortiert werden.
	Pro Tabellenzeile wird ein Tabellen-Element gemacht welches ein Subelement pro Spalte besitzt.
	  -->
	<xsl:template name="table" match="table">
		<xsl:param name="isInInstancePart"/>
		<xsl:variable name="tableId">
			<xsl:call-template name="checkId">
				<xsl:with-param name="id" select="@id"/>
				<xsl:with-param name="isInInstancePart" select="$isInInstancePart"/>
			</xsl:call-template>
		</xsl:variable>
		
		<!--
		In einer Tabelle haben alle Spalten gleich viele Zeilen, 
		daher wird einfach die 1. Spalte für den Loop durch die Tabellenzeilen verwendet
		 -->
		<xsl:for-each select="*[1]/child::node()" >
 			<xsl:element name="{$tableId}">
				<xsl:variable name="rowIndex" select="position()" />
				
				<!--
				Ausgehend von dem Listenelement der 1. Spalte in welchem wir gerade sind
				zuerst zwei Stufen hoch (zu der Tabelle) und dann ein Loop durch sämtliche Spalten machen
				-->
				<xsl:for-each select="../../list | ../../instance-reference-list">
					<xsl:variable name="listId">
						<xsl:call-template name="checkId">
							<xsl:with-param name="id" select="@id"/>
							<xsl:with-param name="isInInstancePart" select="$isInInstancePart"/>
						</xsl:call-template>
					</xsl:variable>
					
					<!-- 
					Mit dem Zeilenindex (aus dem Loop durch die Zeilen der 1. Spalte) den Wert dieser Zeile in der aktuellen Spalte holen.
					 -->
					
					<!-- Dictionary nodes are transformed according to the documentation -->
					<!-- TODO implement the transformation for dictionary nodes inside tables -->
					<xsl:choose>
						<xsl:when test="@entry-type = 'dictionary'">
							<xsl:apply-templates select="child::node()[$rowIndex]">
								<xsl:with-param name="isInInstancePart" select="$isInInstancePart"/>
								<xsl:with-param name="listId" select="$listId"/>
							</xsl:apply-templates>
						</xsl:when>
						<xsl:otherwise>
							<xsl:element name="{$listId}" >
		  						<xsl:value-of select="*[$rowIndex]" />
		 					</xsl:element>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
			</xsl:element>	
		</xsl:for-each>
	</xsl:template>


	<!-- 
	Listen welche direkt (also nicht in eienr Tabelle) im Model deklariert wurden.
	Wie bei Tabellen wird pro Zeile ein List-Element gemacht welches ein Subelement für den Wert hat
	-->
	<xsl:template name="list" match="list">
		<xsl:variable name="listId">
			<xsl:call-template name="checkId">
				<xsl:with-param name="id" select="@id"/>
				<xsl:with-param name="isInInstancePart" select="$false"/>
			</xsl:call-template>
		</xsl:variable>

		<xsl:for-each select="child::node()">
			<xsl:element name="{$listId}" >
				<xsl:value-of select="." />
			</xsl:element>
		</xsl:for-each>
	</xsl:template>


	<!-- dictionary hat nur dict-entry als Subelemnt-->
	<xsl:template name="dictionary" match="dictionary">
	    <xsl:param name="listId" select="@id"/>	
	    
	 	<xsl:apply-templates select="dict-entry">
			<xsl:with-param name="dictionaryId" select="$listId"/>
		</xsl:apply-templates>
	</xsl:template>
	
	<!-- dict-entry das Arrtibut 'key' und das Wert des Knotens werden benötigt  -->
	<xsl:template name="dictEntry" match="dict-entry">
		<xsl:param name="dictionaryId"/>
		
		<xsl:element name="{$dictionaryId}">
			<xsl:element name="{concat($dictionaryId, 'Key')}" ><xsl:value-of select="@key"/></xsl:element>
			<xsl:element name="{concat($dictionaryId, 'Value')}" ><xsl:value-of select="."/></xsl:element>
		</xsl:element>			
	</xsl:template>


	<!-- Einfache Elemente: Das ID-Attribut wird zum Element-Namen, die restlichen Attribute werden verworfen. -->
	<xsl:template name="simpleNode" match="string | text | integer | float | date | boolean | instance-index">
		<xsl:param name="isInInstancePart"/>
		<xsl:variable name="elementId">
			<xsl:call-template name="checkId">
				<xsl:with-param name="id" select="@id"/>
				<xsl:with-param name="isInInstancePart" select="$isInInstancePart"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:element name="{$elementId}" >
			<xsl:value-of select="."/>
		</xsl:element>
	</xsl:template>
	
	
 	<!-- Alles was ich vergessen habe kommt in ein Element 'UNKNOWN_ELEMENT'. Theoretisch dürfte dies nie passieren... -->
 	<xsl:template match="*">
		<xsl:element name="UNKNOWN_ELEMENT">
 			<xsl:copy>
				<xsl:apply-templates select="node()"/>
        	</xsl:copy>
        </xsl:element>	
 	</xsl:template>
 	
 	
  	<!-- 
  	Kleines Hilfstemplate welches prüft, ob das Element ein gültiges ID-Attribut hat.
  	Denn dieses wird nach der Transformation als Element-Name verwendet und muss somit gültig sein!
  	Theoretisch dürfte bei einem gültigen Steuerfall jedoch gar nie ein ungültiges ID-Attribut vorkommen...
  	-->
 	<xsl:template name="checkId">
		<xsl:param name="id"/>
		<xsl:param name="isInInstancePart"/>
		<xsl:choose>			
			<xsl:when test="$id != ''">
				<xsl:choose>
					<!-- Bei Instancen muss zudem der Instance Index aus der ID gelöscht werden! -->
					<xsl:when test="$isInInstancePart = $true and contains($id, '_') ">
						<!-- Den Substring vor dem '_' als Zwischenvariable -->
						<xsl:variable name="vor" select="substring-before($id, '_')"/>
						<!-- Den Instanceindex (die 2 letzten Stellen dieser Zwischenvariable) löschen -->
						<xsl:variable name="ohneIndex" select="substring($vor, 1, (string-length($vor) - 2))"/>
						<!-- Zwischenvariable ohne Instanceindex mit '_' und dem Teil dahinter wieder zusammen basteln... -->
	 					<xsl:value-of select="concat(concat($ohneIndex, '_'), substring-after($id, '_'))"/>
					</xsl:when>				
					<xsl:otherwise>
		  				<xsl:value-of select="$id"/>			
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
 			<xsl:otherwise>UNKNOWN_ID</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
 	
 	
 </xsl:stylesheet>